#include "bouncingScene.h"

//---------------------------------------
bouncingScene::bouncingScene(){

}

//---------------------------------------
void bouncingScene::exit(){

}

//---------------------------------------
void bouncingScene::setupScene(){
	sceneName = "my bouncing scene!";
	addAppListener();
	addMouseListener();
	addKeyboardListener();
	setAutoRender(true);
	
	Akzidenz.loadFont("fonts/AkzidenzGroteskBE-Super.otf", 32);
	
	xml.loadFile("planeSettings.xml");
	
	planePts[0].x = xml.getValue("A:x", 10);
	planePts[0].y = xml.getValue("A:y", 10);
	planePts[1].x = xml.getValue("B:x", 100);
	planePts[1].y = xml.getValue("B:y", 10);
	planePts[2].x = xml.getValue("C:x", 100);
	planePts[2].y = xml.getValue("C:y", 100);	
	planePts[3].x = xml.getValue("D:x", 100);
	planePts[3].y = xml.getValue("D:y", 10);	
	int planeMM	  = xml.getValue("planeMM", 300);	
	
	plane.setup(camW, camH, planeMM, true);
	plane.setMarkerPoints(planePts[0], planePts[1], planePts[2], planePts[3]);
	plane.setMarkerSize(planeMM);
	
	fakeMarker.setup(camW, camH, 60, true);
	
	mx = 0;
	my = 0;
	
	//this is for setting the points with your mouse
	whichPt		 = 0;
	recordingPts = false;
	prevNote  = -1;
	speed	= 0.02;
}


//---------------------------------------
void bouncingScene::update(){
	if(pause)return;
	
	//open up AU LAB to test the audio
	//need to open after OF hence this
	if( ofGetFrameNum() == 30 ){
		//this is for our animation and midi
		//only for mac here - should setup a softsynth for you :)
		//opens AU Lab which should be located in /Developer/Applications/Audio
		//make sure you close AU Lab before you launch the OF app again.
		system("open ../../../data/basicMidi.trak");	
	}
	
	//tie the animation speed to the fps?
	//not such a good idea?
	speed = 0.01 + ( 1.0 / ofGetFrameRate());


	//all this could be moved into a seperate class!!!
	//like CARD TRAVELLER?

	//this happens at the begining
	int numFound = currentCards.size();
	if( basicShape.getState() == 0 && numFound >= 2){
		srcID = currentCards[0].id;
		dstID = currentCards[1].id;
		playingCard = currentCards[0];
		nextCard = currentCards[1];
		basicShape.begin(currentCards[0].center, currentCards[1].center);
	}	
		
	//find the closest matching card that isn't the one we were just at
	if(basicShape.getState() == 2 && numFound >= 2){
		ofxPoint2f currPos = basicShape.pos;		
		float closestDist = 999999;
		int closestID = -1;
		ofxVec2f vec;
		for(int i = 0; i < numFound; i++){
			if(  currentCards[i].id == dstID ) continue;
			if( numFound > 2 && currentCards[i].id == srcID) continue;
			if( !currentCards[i].cached == false )continue; //we don't want fresh off the boat cards
			
			vec =  (currentCards[i].center - basicShape.pos);
			float dist = vec.length();
					
			if(dist < closestDist){
				closestDist = dist;
				closestID = i;
				
				//some variation so the pattern doesn't just repeat :)
				//OPTIMIZE this should be scaled to the number of cards
				//there are as - with a ton of cards it ends up not finding the neareast very often
				//or it should start from a random position in the array?  better maybe?
				if( (int)ofRandom(0, 13) == 3 )break;
			}
		}
		
		//if we are at are target - lets play a sound based on out position
		//then start our new journey
		if( closestID > -1){
			//printf("closest found is %i\n", closestID);
			
			ofxPoint2f pt = basicShape.dst;

			//turn the old note off
			if( prevNote != -1)midi.sendNoteOff(prevNote, 100);
			
			//some notes in the c scale
			int notes[28] = {36, 38, 40, 41, 43, 45, 47, 48, 50, 52, 53, 55, 57, 59, 60, 62, 64, 65, 67, 69, 71, 72, 74, 76, 77, 79, 81, 83};
			
			playingNumber = 1 + (nextCard.getCard() % 13);
			
			//float pitch =  1.0 - ( pt.y  / ofGetHeight() );
			//lets try and make the pitch based on the number of the card
			float pitch =  (float)playingNumber / 14.1;
			pitch *= 28;
			
			midi.sendControlChange(1, 10, ofRandom(-127, 127));
			
			prevNote = notes[(int)pitch];
			midi.sendNoteOn(prevNote, 100);
		
			basicShape.begin( currentCards[closestID].center );
			srcID = dstID;
			dstID = currentCards[closestID].id;
			playingCard = nextCard;
			nextCard = currentCards[closestID];		
			
			playingCard.computeSqPoints();
			fakeMarker.setMarkerPoints(playingCard.sqPts[0], playingCard.sqPts[1], playingCard.sqPts[2], playingCard.sqPts[3]);
		} 
		
	}
			
	//finally update the animation
	if( basicShape.getState() == 1) basicShape.update(speed);

}

//---------------------------------------
void bouncingScene::render(){
	if( hide || pause ) return;
	
	ofEnableAlphaBlending();
	
	//draw the shadow of the number
	if( fakeMarker.isMarkerSet() && currentCards.size() > 0){
		fakeMarker.loadARMatrix();
			glPushMatrix();
				float scale = basicShape.pct * 0.3;
				glScalef(1 + scale, -1 - scale, 1 + scale);
				ofSetColor(0, 0, 0, 200 - basicShape.pct * 80);	
				Akzidenz.drawString( ofToString(playingNumber),-Akzidenz.stringWidth(ofToString(playingNumber))/2, 10);
			glPopMatrix();
	}
	
	//draw the bouncer
	if( plane.isMarkerSet() && currentCards.size() > 0){
		plane.loadARMatrix();
			ofSetColor(0, 0, 0, 140);
			
			ofxPoint2f pos;
			pos = plane.screenToMarker( basicShape.pos.x, basicShape.pos.y );
			ofCircle(pos.x, pos.y, 12);
			
			ofxPoint2f pt1;
			pt1 = plane.screenToMarker( basicShape.src.x, basicShape.src.y );
			
			ofxPoint2f pt2;
			pt2 = plane.screenToMarker( basicShape.dst.x, basicShape.dst.y );
			
			ofxPoint2f mid;
			mid = pt1 * 0.5 + pt2 * 0.5;
		
			ofxVec2f delta;
			delta = pt2 - pt1;
			delta.normalize();
			
			int numTrail = 80;
			for(int i = 0; i < numTrail; i++){
				glPushMatrix();
					float pctMod = (float) i / (float)numTrail;

					glTranslatef(mid.x, mid.y, 0);
					glRotatef(pctMod * basicShape.pct * 180, -delta.y, delta.x, 0);
				
					if( i != numTrail-1 ) ofSetColor(255, 255, 0, 23);
					else ofSetColor(00, 55, 230, 160);

					ofCircle(pt1.x - mid.x, pt1.y - mid.y, 11  * pctMod);
								
				glPopMatrix();
			}
		
		plane.unloadARMatrix();
	}
	
	//draw the number
	if( fakeMarker.isMarkerSet() && currentCards.size() > 0){
		fakeMarker.loadARMatrix();
			glPushMatrix();
				glScalef(1, -1, 1);			
				glTranslatef(0, 0, basicShape.pct * 110);
				ofSetColor(200, 244, 10, 220);
				Akzidenz.drawString( ofToString(playingNumber), -Akzidenz.stringWidth(ofToString(playingNumber))/2, 10);
			glPopMatrix();
		fakeMarker.unloadARMatrix();
	}	

	ofDisableAlphaBlending();

	renderGui();
}

//---------------------------------------
void bouncingScene::renderGui(){

	if(recordingPts){
		ofNoFill();
			ofSetColor(255, 0, 130);
			ofBeginShape();
				for(int i = 0; i < whichPt; i++){
					ofVertex(planePts[i].x, planePts[i].y);
				}
			ofEndShape(false);
			for(int i = 0; i < whichPt; i++){
				ofCircle(planePts[i].x, planePts[i].y, 4);
			}
		ofFill();
	}
	
	if(debug){
		drawTxtMsg("scene is: "+sceneName+" r to record points - s to save settings", 0, camH - 40);
		drawDebugTxt(0, camH - 20);	
	}
}

//---------------------------------------		
void bouncingScene::keyPressed( int key ){
	if(pause)return;

	if(key == 'r'){
		recordingPts = true;
		whichPt = 0;
		debugTxt = "set plane points - click corners clockwise from top left";		
	}

	if( key == 's'){
		xml.setValue("A:x", planePts[0].x);
		xml.setValue("A:y", planePts[0].y);
		xml.setValue("B:x", planePts[1].x);
		xml.setValue("B:y", planePts[1].y);
		xml.setValue("C:x", planePts[2].x);
		xml.setValue("C:y", planePts[2].y);	
		xml.setValue("D:x", planePts[3].x);
		xml.setValue("D:y", planePts[3].y);
		xml.saveFile("planeSettings.xml");
		debugTxt = "plane settings saved";
	}
}

//---------------------------------------
void bouncingScene::keyReleased( int key ){
	if(pause)return;
	
}
	
//---------------------------------------
void bouncingScene::mouseMoved(int x, int y ){
	if(pause)return;
	
	mx = x;
	my = y;
}

//---------------------------------------
void bouncingScene::mouseDragged( int x, int y, int button ){
	if(pause)return;

	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
	}
}

//---------------------------------------
void bouncingScene::mousePressed( int x, int y, int button ){
	if(pause)return;

	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
	}
}

//---------------------------------------
void bouncingScene::mouseReleased(int x, int y, int button ){
	if(pause)return;	
	
	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
		whichPt++;
		if( whichPt == 4){
			recordingPts = false;
			debugTxt = "points set - hit s key to save";
			plane.setMarkerPoints(planePts[0], planePts[1], planePts[2], planePts[3]);
		}
	}
	printf("mouse released is %i %i\n", x, y);
}	

