#include "sonarScene.h"

//---------------------------------------
sonarScene::sonarScene(){

}

//---------------------------------------
void sonarScene::setupScene(){
	sceneName = "sonarScene";
	addAppListener();
	addMouseListener();
	addKeyboardListener();
	setAutoRender(true);
	
	//load the plane from xml
	xml.loadFile("planeSettings.xml");
	
	planePts[0].x = xml.getValue("A:x", 10);
	planePts[0].y = xml.getValue("A:y", 10);
	planePts[1].x = xml.getValue("B:x", 100);
	planePts[1].y = xml.getValue("B:y", 10);
	planePts[2].x = xml.getValue("C:x", 100);
	planePts[2].y = xml.getValue("C:y", 100);	
	planePts[3].x = xml.getValue("D:x", 100);
	planePts[3].y = xml.getValue("D:y", 10);	
	int planeMM	  = xml.getValue("planeMM", 300);	
	
	plane.setup(camW, camH, planeMM, true);
	plane.setMarkerPoints(planePts[0], planePts[1], planePts[2], planePts[3]);
	plane.setMarkerSize(planeMM);
	
	mx = 0;
	my = 0;
	
	//this is for setting the points with your mouse
	whichPt		 = 0;
	recordingPts = false;
	
	
	angleOfScanner = 0;
	
}

//---------------------------------------
void sonarScene::render(){
	if( hide || pause ) return;
	
	//draw a circle for each card found
	if( plane.isMarkerSet()){
		plane.loadARMatrix(ofGetWidth(), ofGetHeight());
		
			ofSetColor(0x00FF00);
			
			if( currentCards.size() > 0){
				for(int i = 0; i < currentCards.size(); i++){
					ofPoint pt = plane.screenToMarker(currentCards[i].center.x, currentCards[i].center.y);
					glPushMatrix();
					// checking AR stuff, damn !! 
					// not gaurenteed order! -- > fails glTranslatef(0,0,100*fabs(sin(ofGetElapsedTimef() + i*0.1f)));
					//glTranslatef(0,0,100*MAX((1-currentCards[i].activity), 0));
					ofCircle(pt.x, pt.y,  4);
					glPopMatrix();
				}
			}
			
			glPushMatrix();
				ofSetColor(255,255,255);
		
				ofPoint pt;
				pt.set(320,240);
				pt = plane.screenToMarker(pt.x, pt.y);
				glTranslatef(0,	0, 0);
				ofLine(pt.x, pt.y,pt.x + 100 * cos(angleOfScanner), pt.y + 100 * sin(angleOfScanner));
			glPopMatrix();
		
		
		plane.unloadARMatrix();
		
		ofSetColor(0,255,255) ; //, <#int g#>, <#int b#>)
		if( currentCards.size() > 0){
			for(int i = 0; i < currentCards.size(); i++){
				ofPoint pt2(currentCards[i].center.x, currentCards[i].center.y);
				glPushMatrix();
				// checking AR stuff, damn !! 
				// not gaurenteed order! -- > fails glTranslatef(0,0,100*fabs(sin(ofGetElapsedTimef() + i*0.1f)));
				//glTranslatef(0,0,100*MAX((1-currentCards[i].activity), 0));
				ofCircle(pt2.x, pt2.y,  4);
				glPopMatrix();
			}
		}
		
		
	}
	
	//show the plane points we are marking out
	if(recordingPts){
		ofNoFill();
			ofSetColor(255, 0, 130);
			ofBeginShape();
				for(int i = 0; i < whichPt; i++){
					ofVertex(planePts[i].x, planePts[i].y);
				}
			ofEndShape(false);
			for(int i = 0; i < whichPt; i++){
				ofCircle(planePts[i].x, planePts[i].y, 4);
			}
		ofFill();
	}
	
	//show our debug text
	if(debug){
		drawTxtMsg("scene is: "+sceneName+" r to record points - s to save settings", 0, camH - 40);
		drawDebugTxt(0, camH - 20);	
	}

}

//---------------------------------------
void sonarScene::update(){
	if( pause ) return;
	
		float prevAngle = angleOfScanner;
		angleOfScanner += 0.01f;
		if (angleOfScanner > TWO_PI) angleOfScanner -= TWO_PI;
	
	
	if( currentCards.size() > 0){
		for(int i = 0; i < currentCards.size(); i++){
			ofPoint pt;
			pt.set(currentCards[i].center.x, currentCards[i].center.y,0);
			float dx = pt.x - 320;
			float dy = pt.y - 240;
			float angle = -(atan2(dy, dx));
			
			if (angle > TWO_PI) angle -= TWO_PI;
			//if (diff < -PI) diff += TWO_PI;
			
			//while (angle < (prevAngle-PI)) angle += TWO_PI; // ok?
					
			float diff = angle - angleOfScanner;
			if (diff > PI) diff -= TWO_PI;
			if (diff < -PI) diff += TWO_PI;
			
		
			
			if (fabs(diff) < 0.07){
				printf ("got card %i -- %f\n", i, fabs(diff));
			}
		}
	}
	

}

//---------------------------------------		
void sonarScene::keyPressed( int key ){
	if( pause ) return;

	if(key == 'r'){
		recordingPts = true;
		whichPt = 0;
		debugTxt = "set plane points - click corners clockwise from top left";		
	}

	if( key == 's'){
		xml.setValue("A:x", planePts[0].x);
		xml.setValue("A:y", planePts[0].y);
		xml.setValue("B:x", planePts[1].x);
		xml.setValue("B:y", planePts[1].y);
		xml.setValue("C:x", planePts[2].x);
		xml.setValue("C:y", planePts[2].y);	
		xml.setValue("D:x", planePts[3].x);
		xml.setValue("D:y", planePts[3].y);
		xml.saveFile("planeSettings.xml");
		debugTxt = "plane settings saved";
	}
}

//---------------------------------------
void sonarScene::keyReleased( int key ){
	if( pause ) return;

}
	
//---------------------------------------
void sonarScene::mouseMoved(int x, int y ){
	if( pause ) return;

	mx = x;
	my = y;
}

//---------------------------------------
void sonarScene::mouseDragged( int x, int y, int button ){

	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
	}
}

//---------------------------------------
void sonarScene::mousePressed( int x, int y, int button ){
	if( pause ) return;

	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
	}
}

//---------------------------------------
void sonarScene::mouseReleased(int x, int y, int button ){
	if( pause ) return;
	
	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
		whichPt++;
		if( whichPt == 4){
			recordingPts = false;
			debugTxt = "points set - hit s key to save";
			plane.setMarkerPoints(planePts[0], planePts[1], planePts[2], planePts[3]);
		}
	}

}	

